"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var observable_1 = require("tns-core-modules/data/observable");
var rxjs_1 = require("rxjs");
var Common = (function (_super) {
    __extends(Common, _super);
    function Common() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.isConnected = new rxjs_1.BehaviorSubject(null);
        _this.methods = {};
        _this.callbacks = {};
        _this.recordSeparator = String.fromCharCode(0x1e);
        _this.roomId = 0;
        _this.socketUrl = '';
        _this.protocol = {
            name: 'json',
            transferFormat: 1,
            version: 1,
            writeMessage: function (message) {
                return TextMessageFormat.write(JSON.stringify(message));
            }
        };
        return _this;
    }
    Common.prototype.start = function (httpURL) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            var run = function () {
                _this.socketUrl = httpURL.replace(/(http)(s)?\:\/\//, 'ws$2://');
                _this.socketUrl += '?id=';
                var self = _this;
                _this.makeRequest('POST', httpURL + "/negotiate", function (err, data) {
                    if (err) {
                        reject(err);
                        return false;
                    }
                    else {
                        var connId = _this.socketUrl;
                        if (typeof data === 'object') {
                            connId = data.connectionId;
                        }
                        else {
                            var _data = JSON.parse(data);
                            connId = _data.connectionId;
                        }
                        _this.socketUrl += connId;
                        return self.openSocketConnection(_this.socketUrl)
                            .then(function (res) {
                            if (res) {
                                _this.isConnected.next(true);
                                resolve(true);
                            }
                        });
                    }
                });
            };
            if (_this.websocket && _this.websocket.readyState && _this.websocket.readyState !== WebSocketStatus.Closed) {
                _this.close();
                _this.isConnected.next(false);
                reject('Error');
                return run();
            }
            else {
                return run();
            }
        });
    };
    Common.prototype.openSocketConnection = function (socketUrl) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            _this.websocket = new WebSocket(socketUrl, _this.recordSeparator[JSON.stringify(_this.protocol)]);
            _this.websocket.onopen = function (event) {
                _this.websocket.send(JSON.stringify({ 'protocol': 'json', 'version': 1 }));
                _this.websocket.send(_this.recordSeparator);
            };
            _this.websocket.onmessage = function (data) { return _this._onMessage(data); };
            _this.websocket.onclose = _this.close();
            _this.websocket.onerror = function (err) { return reject(err); };
            resolve(_this.websocket);
        });
    };
    Common.prototype.close = function () {
        if (this.websocket) {
            this.websocket.close();
            this.isConnected.next(false);
        }
    };
    Common.prototype.on = function (methodName, newMethod) {
        if (!methodName || !newMethod) {
            return;
        }
        methodName = methodName.toLowerCase();
        if (!this.methods[methodName]) {
            this.methods[methodName] = [];
        }
        if (this.methods[methodName].indexOf(newMethod) !== -1) {
            return;
        }
        this.methods[methodName].push(newMethod);
    };
    Common.prototype.invoke = function (methodName) {
        var _this = this;
        var args = [];
        for (var _a = 1; _a < arguments.length; _a++) {
            args[_a - 1] = arguments[_a];
        }
        for (var _i = 1; _i < arguments.length; _i++) {
            args[_i - 1] = arguments[_i];
        }
        var invocationDescriptor = this.createInvocation(methodName, args, false);
        var _invocationEvent = null;
        return new Promise(function (resolve, reject) {
            _this.callbacks[invocationDescriptor.invocationId] = function (invocationEvent, error) {
                if (error) {
                    reject(error);
                    return;
                }
                else if (invocationEvent) {
                    if (invocationEvent.type === MessageType.Completion) {
                        if (invocationEvent.error) {
                            reject(new Error(invocationEvent.error));
                        }
                        else {
                            resolve(invocationEvent.result);
                        }
                    }
                    else {
                        reject(new Error('Unexpected message type: ' + invocationEvent.type));
                    }
                }
            };
            var message = _this.protocol.writeMessage(invocationDescriptor);
            _this.websocket.send(message);
            resolve({ data: TextMessageFormat.parse(message), date: new Date() });
        });
    };
    Common.prototype.createInvocation = function (methodName, args, nonblocking) {
        if (nonblocking) {
            return {
                arguments: args,
                target: methodName,
                type: MessageType.Invocation,
            };
        }
        else {
            var id = this.roomId;
            this.roomId++;
            return {
                arguments: args,
                invocationId: id.toString(),
                target: methodName,
                type: MessageType.Invocation,
            };
        }
    };
    Common.prototype._onMessage = function (event) {
        var data = event.data, target = event.target;
        var parseData = TextMessageFormat.parse(data);
        var parseTarget = JSON.parse(parseData[0]);
        if (parseTarget && parseTarget['type'] && parseTarget['type'] !== 3) {
            if (parseTarget['type'] === MessageType.Invocation && parseTarget.target && this.methods[parseTarget.target.toLowerCase()]) {
                this.methods[parseTarget.target.toLowerCase()][0](parseTarget);
            }
        }
    };
    Common.prototype.makeRequest = function (method, url, done) {
        var xhr = new XMLHttpRequest();
        xhr.open(method, url);
        xhr.onload = function () {
            done(null, xhr.response);
        };
        xhr.onerror = function () {
            done(xhr.response);
        };
        xhr.send();
    };
    return Common;
}(observable_1.Observable));
exports.Common = Common;
var WebSocketStatus;
(function (WebSocketStatus) {
    WebSocketStatus[WebSocketStatus["Connecting"] = 0] = "Connecting";
    WebSocketStatus[WebSocketStatus["Open"] = 1] = "Open";
    WebSocketStatus[WebSocketStatus["Closing"] = 2] = "Closing";
    WebSocketStatus[WebSocketStatus["Closed"] = 3] = "Closed";
})(WebSocketStatus = exports.WebSocketStatus || (exports.WebSocketStatus = {}));
var MessageType;
(function (MessageType) {
    MessageType[MessageType["Invocation"] = 1] = "Invocation";
    MessageType[MessageType["StreamItem"] = 2] = "StreamItem";
    MessageType[MessageType["Completion"] = 3] = "Completion";
    MessageType[MessageType["StreamInvocation"] = 4] = "StreamInvocation";
    MessageType[MessageType["CancelInvocation"] = 5] = "CancelInvocation";
    MessageType[MessageType["Ping"] = 6] = "Ping";
    MessageType[MessageType["Close"] = 7] = "Close";
})(MessageType = exports.MessageType || (exports.MessageType = {}));
var TextMessageFormat = (function () {
    function TextMessageFormat() {
    }
    TextMessageFormat.write = function (output) {
        return "" + output + TextMessageFormat.RecordSeparator;
    };
    TextMessageFormat.parse = function (input) {
        if (input[input.length - 1] !== TextMessageFormat.RecordSeparator) {
            throw new Error('incomplete.');
        }
        var messages = input.split(TextMessageFormat.RecordSeparator);
        messages.pop();
        return messages;
    };
    TextMessageFormat.RecordSeparatorCode = 0x1e;
    TextMessageFormat.RecordSeparator = String.fromCharCode(TextMessageFormat.RecordSeparatorCode);
    return TextMessageFormat;
}());
exports.TextMessageFormat = TextMessageFormat;
//# sourceMappingURL=signalr-core.common.js.map